library(coda)
library(rjags)
    load.module('glm')
library(foreach)
    library(doMC); registerDoMC()

# j = go.real(model.ordinal.manip_only, ss(sb, tv == 2 & punish.nc != "Relent"))
# j2 = go.real(model.ordinal.mgai, ss(sb, tv == 2 & (is.na(punish.nc) | punish.nc == "Yes")), n.chains = 4, n.adapt = 1500, n.sample = 500)
# dev.set(2); qplot(x = y, y = ..density.., fill = factor(protect_kids), data = as.data.frame(j2$jags[[1]]$data()[c("y", "protect_kids")]), geom = "histogram", binwidth = .5, position = "dodge") + scale_x_continuous(breaks = 1:9) + guides(fill = F) + scale_y_continuous(breaks = seq(0, .8, .1), limits = c(0, .8))
# dev.set(3); qplot(x = v, y = ..density.., fill = f, data = fframe(j2$jsamp$yhat[3,], j2$jsamp$yhat[1,]), geom = "histogram", binwidth = .5, position = "dodge") + scale_x_continuous(breaks = 1:9) + guides(fill = F) + scale_y_continuous(breaks = seq(0, .8, .1), limits = c(0, .8))

. = quote

mkmodel.normal = function(prior, mu) as.bugs.model(
   {.(prior)
    sd ~ dunif(0, 100)

    for (s in 1 : N)
       {y[s] ~ dnorm(.(mu), sd^(-2))
        yhat[s] ~ dnorm(.(mu), sd^(-2))}})
#        y[s] ~ dround(y0[s], 0)}})

mkmodel.ordinal = function(prior, mu) as.bugs.model(
   {.(prior)

    # Set up thresholds.
    threshold1 ~ dnorm(0, 100^(-2))
    thresholds[1] <- threshold1
    for (cat in 1 : (ncats - 2))
        {t_diffs[cat] ~ dunif(0, 1000)
         thresholds[cat + 1] <- thresholds[cat] + t_diffs[cat]}

    for (s in 1 : N)
       {mu[s] <- .(mu)

        # Compute the probability of each category.
        q[s, 1] <- phi(thresholds[1] - mu[s])
        p[s, 1] <- q[s, 1]
        for (cat in 2 : (ncats - 1))
           {q[s, cat] <- phi(thresholds[cat] - mu[s])
            p[s, cat] <- q[s, cat] - q[s, cat - 1]}
        p[s, ncats] <- 1 - q[s, ncats - 1]

        # Draw y from the categorical distribution.
        y[s] ~ dcat(p[s,])
        yhat[s] ~ dcat(p[s,])}})

model.ordinal.manip_only = list(
    bugs = mkmodel.ordinal(
        prior = .(b.protect_kids ~ dnorm(0, 100^(-2))),
        mu = .(b.protect_kids * protect_kids[s])),
    jags.sample.monitor = qw(b.protect_kids, thresholds, yhat),
    mk.inits = function (dat) list(
        b.protect_kids = runif(1, -2, 2),
        threshold1 = runif(1, -1, 3),
        t_diffs = rlnorm(dat$ncats - 2, -1, 0.25)),
    mk.data.real = function (sb) with(sb, list(
        N = nrow(sb),
        ncats = length(levels(oc.fine)),
        protect_kids = punish.nc == "Punish",
        y = as.integer(oc.fine))),
    mk.data.fake = function()
       {b.protect_kids = 1
        N = 300
        ncats = 4
        protect_kids = c(rep(0, N/2), rep(1, N/2))
        y0 = rnorm(N, b.protect_kids * protect_kids, 1)
        thresholds = c(-1, 0, .5)
        t_diffs = diff(thresholds)
        y = ifelse(
                    y0 < thresholds[1], 1,
            ifelse( y0 < thresholds[2], 2,
            ifelse( y0 < thresholds[3], 3,
                           4)))
        punl(protect_kids, y, N, ncats)})

model.ordinal.mga = list(
    bugs = mkmodel.ordinal(
        prior = .(
           {b.protect_kids ~ dnorm(0, 100^(-2))
            b.female ~ dnorm(0, 100^(-2))
            b.age ~ dnorm(0, 100^(-2))}),
        mu = .(
            b.protect_kids * protect_kids[s] +
            b.female * female[s] +
            b.age * age[s])),
    jags.sample.monitor = qw(b.protect_kids, b.female, b.age, thresholds, yhat),
    mk.inits = function (dat) list(
        b.protect_kids = runif(1, -2, 2),
        b.female = runif(1, -2, 2),
        b.age = runif(1, -.2, .2),
        threshold1 = runif(1, -3, 3),
        t_diffs = rlnorm(dat$ncats - 2, -1, 0.25)),
    mk.data.real = function (sb) with(sb, list(
        N = nrow(sb),
        ncats = length(levels(oc.fine)),
        protect_kids = punish.nc == "Punish",
        female = as.integer(gender == "Female"),
        age = as.integer(age - 18),
        y = as.integer(oc.fine))),
    mk.data.fake = function()
       {b.protect_kids = 1
        b.female = -.5
        b.age = .01
        N = 300
        ncats = 4
        protect_kids = c(rep(0, N/2), rep(1, N/2))
        female = rep(c(0, 1), N/2)
        age = round(seq(18, 60, len = N)) - 18
        y0 = rnorm(N, sd = 1,
            b.protect_kids * protect_kids +
            b.female * female +
            b.age * age)
        thresholds = c(-1, 0, .5)
        t_diffs = diff(thresholds)
        y = ifelse(
                    y0 < thresholds[1], 1,
            ifelse( y0 < thresholds[2], 2,
            ifelse( y0 < thresholds[3], 3,
                           4)))
        punl(protect_kids, female, age, y, N, ncats)})

model.ordinal.mgai = list(
    bugs = mkmodel.ordinal(
        prior = .(
           {b.protect_kids ~ dnorm(0, 100^(-2))
            b.female ~ dnorm(0, 100^(-2))
            b.protfem ~ dnorm(0, 100^(-2))
            b.age ~ dnorm(0, 100^(-2))}),
        mu = .(
            b.protect_kids * protect_kids[s] +
            b.female * female[s] +
            b.protfem * female[s] * protect_kids[s] +
            b.age * age[s])),
    jags.sample.monitor = qw(b.protect_kids, b.female, b.protfem, b.age, thresholds, yhat),
    mk.inits = function (dat) list(
        b.protect_kids = runif(1, -1.5, 1.5),
        b.female = runif(1, -1.5, 1.5),
        b.protfem = runif(1, -1.5, 1.5),
        b.age = runif(1, -.1, .1),
        threshold1 = runif(1, -3, 3),
        t_diffs = rlnorm(dat$ncats - 2, -1, 0.25)),
    mk.data.real = function (sb) with(sb, list(
        N = nrow(sb),
        ncats = length(levels(oc.fine)),
        protect_kids = punish.nc == "Punish",
        female = as.integer(gender == "Female"),
        age = as.integer(age - 18),
        y = as.integer(oc.fine))),
    mk.data.fake = function()
       {b.protect_kids = 1
        b.female = -.5
        b.protfem = .5
        b.age = .01
        N = 300
        ncats = 4
        protect_kids = c(rep(0, N/2), rep(1, N/2))
        female = rep(c(0, 1), N/2)
        age = round(seq(18, 60, len = N)) - 18
        y0 = rnorm(N, sd = 1,
            b.protect_kids * protect_kids +
            b.female * female +
            b.protfem * female * protect_kids +
            b.age * age)
        thresholds = c(-1, 0, .5)
        t_diffs = diff(thresholds)
        y = ifelse(
                    y0 < thresholds[1], 1,
            ifelse( y0 < thresholds[2], 2,
            ifelse( y0 < thresholds[3], 3,
                           4)))
        punl(protect_kids, female, age, y, N, ncats)})

model.normal.manip_only = list(
    bugs = mkmodel.normal(
        prior = .(
           {b0 ~ dnorm(0, 100^(-2))
            b.protect_kids ~ dnorm(0, 100^(-2))}),
        mu = .(
            b0 + b.protect_kids * protect_kids[s])),
    jags.sample.monitor = qw(sd, b0, b.protect_kids, yhat),
    mk.inits = function (dat) list(
        sd = rlnorm(1, -1, 2),
        b0 = runif(1, -10, 10),
        b.protect_kids = runif(1, -10, 10)),
    mk.data.real = function (sb) with(sb, list(
        N = nrow(sb),
        protect_kids = punish.nc == "Punish",
        y = as.integer(oc.fine))),
    mk.data.fake = function()
        modifyList(model.ordinal.manip_only$mk.data.fake(),
            list(ncats = NULL)))

go.test = function(model, seed = 1, n.chains = 4, n.adapt = 200, n.sample = 500)
   {set.seed(seed)
    dat = model$mk.data.fake()
    inits = replicate(n.chains, simp = F,
        model$mk.inits(dat))
    multijags(model$bugs,
        data = dat,
        inits = inits,
        monitor = names(inits[[1]]),
        jags.sample.monitor = model$jags.sample.monitor,
        n.adapt = n.adapt,
        n.sample = n.sample)}

go.real = function(model, sb, n.chains = 4, n.adapt = 200, n.sample = 500)
   {dat = model$mk.data.real(sb)
    inits = replicate(n.chains, simp = F,
        model$mk.inits(dat))
    multijags(model$bugs,
        data = dat,
        inits = inits,
        monitor = names(inits[[1]]),
        jags.sample.monitor = model$jags.sample.monitor,
        n.adapt = n.adapt,
        n.sample = n.sample)}
        #n.nonadapt = 200,

distribution.plot = function(j)
   {d = table(rbind(
        data.frame(type = "observed",
            protect_kids = ifelse(j$data$protect_kids, "Punish", "NoOpp"),
            y = j$data$y),
        data.frame(type = "predicted", protect_kids = "NoOpp",
            y = c(j$jsamp$yhat[ which(!j$data$protect_kids), ])),
        data.frame(type = "predicted", protect_kids = "Punish",
            y = c(j$jsamp$yhat[ which(j$data$protect_kids), ]))))
    d = as.data.frame(prop.table(d, which(names(dimnames(d)) == "type")))
    ggplot() +
        geom_bar(
            aes(y, Freq, fill = protect_kids),
            data = ss(d, type == "predicted"),
            stat = "identity", position = "dodge") +
        geom_linerange(
            aes(x = y,
                ymin = pmin(predicted, observed),
                ymax = pmax(predicted, observed),
                group = protect_kids),
            data = dcast(d, y + protect_kids ~ type, value.var = "Freq"),
            position = position_dodge(width = 1),
            size = 2) +
        scale_x_discrete(
            name = "oc.fine",
            labels = levels(sb$oc.fine)) +
        scale_y_continuous("proportion",
            expand = c(0, 0), limits = c(0, .18)) +
        #theme(panel.grid.major.x = element_blank())}
        no.gridlines()}

vulnjags.mk.data = function(sb) with(sb, punl(
    N.subjects = nrow(sb),
    vuln.s, vuln.adult, vuln.10yo,
    censor = censor == "Censor"))
#    female = gender == "Female",
#    age = age - 18))

vulnjags = function(model, sb = NULL,
        n.chains = 4, n.adapt = 5000, n.sample = 100 * thin, thin = 10)
   {dat = vulnjags.mk.data(sb)
    inits = replicate(n.chains, simp = F, model$mk.inits())
    multijags(model$bugs,
        data = dat,
        inits = inits,
        monitor = names(inits[[1]]),
        jags.sample.monitor = model$jags.sample.monitor,
        n.adapt = n.adapt,
        n.sample = n.sample,
        thin = thin)}

coda.qmean = function(mcmc.list, true.vals = NULL)
  {m = signif(digits = 3, mapcols(
       do.call(rbind, mcmc.list), qmean))
   if (!is.null(true.vals))
      {m = rbind(m[c("hi", "mean"),],
           true = as.numeric(true.vals[dimnames(m)[[2]]]),
           m["lo",, drop = F])}
   as.data.frame(m)}

vuln.jags2df = function(j)
   {d = do.call(rbind, lapply(qw(vuln.s.pred, vuln.adult.pred, vuln.10yo.pred), function(v)
        transform(
            melt(j$jsamp[[v]],
                varnames = qw(subject, i), value.name = "y"),
            target = str_replace(v, "\\.pred$", ""),
            type = "pred")))
    d = do.call(rbind, c(list(d), lapply(qw(vuln.s, vuln.adult, vuln.10yo),
        function(v) data.frame(
            subject = 1 : j$data$N.subjects,
            i = NA,
            y = j.tv3.m1.real$data[[v]],
            target = v,
            type = "real"))))}

vulnjags.ppc = function(d, f)
    ggplot(aggregate(y ~ i, data = ss(d, type == "pred"), FUN = f)) +
        geom_densitybw(aes(y)) +
        geom_vline(aes_string(xintercept = f(ss(d, type == "real")$y)),
            color = "red")
